@@ -1,5 +1,7 @@
 use crate::prelude::*;
 use image::GrayImage;
+use itertools::izip;
+use rayon::prelude::*;
 
 /// see https://www.itu.int/rec/T-REC-T.871
 fn rgb_to_yuv(rgb: &[f32; 3]) -> [f32; 3] {
@@ -28,12 +30,19 @@ impl Decompose for RgbImage {
         let mut red = GrayImage::new(self.width(), self.height());
         let mut green = red.clone();
         let mut blue = red.clone();
-        Window::from_image(&red).iter_pixels().for_each(|p| {
-            let data = self.get_pixel(p.0, p.1);
-            red.put_pixel(p.0, p.1, Luma([data[0]]));
-            green.put_pixel(p.0, p.1, Luma([data[1]]));
-            blue.put_pixel(p.0, p.1, Luma([data[2]]));
+        izip!(
+            red.pixels_mut(),
+            green.pixels_mut(),
+            blue.pixels_mut(),
+            self.pixels()
+        )
+        .par_bridge()
+        .for_each(|(r, g, b, rgb)| {
+            *r = Luma([rgb[0]]);
+            *g = Luma([rgb[1]]);
+            *b = Luma([rgb[2]]);
         });
+
         [red, green, blue]
     }
 
@@ -41,32 +50,37 @@ impl Decompose for RgbImage {
         let mut y = GrayImage::new(self.width(), self.height());
         let mut u = y.clone();
         let mut v = y.clone();
-        Window::from_image(&y).iter_pixels().for_each(|p| {
-            let data = self.get_pixel(p.0, p.1);
-            let yuv = rgb_to_yuv(&data.0.map(|c| c as f32));
-            y.put_pixel(p.0, p.1, Luma([yuv[0].clamp(0., 255.) as u8]));
-            u.put_pixel(p.0, p.1, Luma([yuv[1].clamp(0., 255.) as u8]));
-            v.put_pixel(p.0, p.1, Luma([yuv[2].clamp(0., 255.) as u8]));
+        izip!(
+            y.pixels_mut(),
+            u.pixels_mut(),
+            v.pixels_mut(),
+            self.pixels()
+        )
+        .par_bridge()
+        .for_each(|(y, u, v, rgb)| {
+            let yuv = rgb_to_yuv(&rgb.0.map(|c| c as f32));
+            *y = Luma([yuv[0].clamp(0., 255.) as u8]);
+            *u = Luma([yuv[1].clamp(0., 255.) as u8]);
+            *v = Luma([yuv[2].clamp(0., 255.) as u8]);
         });
+
         [y, u, v]
     }
 }
 
 pub fn merge_similarity_channels(input: &[&GraySimilarityImage; 3]) -> RGBSimilarityImage {
     let mut output = RGBSimilarityImage::new(input[0].width(), input[0].height());
-    Window::new((0, 0), (output.width() - 1, output.height() - 1))
-        .iter_pixels()
-        .for_each(|p| {
-            output.put_pixel(
-                p.0,
-                p.1,
-                Rgb([
-                    input[0].get_pixel(p.0, p.1)[0],
-                    input[1].get_pixel(p.0, p.1)[0],
-                    input[2].get_pixel(p.0, p.1)[0],
-                ]),
-            )
-        });
+    izip!(
+        input[0].pixels(),
+        input[1].pixels(),
+        input[2].pixels(),
+        output.pixels_mut()
+    )
+    .par_bridge()
+    .for_each(|p| {
+        *p.3 = Rgb([p.0[0], p.1[0], p.2[0]]);
+    });
+
     output
 }
 
